﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CommonStorage.Model
{
    /// <summary>
    /// 报表配置
    /// </summary>
    [Table("report_configs")]
    public  class ReportConfig
    {
        /**[JsonConverter(typeof(LongToStringConverter))]*/
        /// <summary>
        /// ID
        /// </summary>
        [Key]
        public long ID { get; set; }

        /// <summary>
        /// 产品代号
        /// </summary>
        public string ProductCode { get; set; }

        /// <summary>
        /// 报告名称
        /// </summary>
        public string ReportName { get; set; }

        /// <summary>
        /// 导出报告的软件模块的名称
        /// </summary>
        public string AppName { get; set; }
        /// <summary>
        /// 报告模板文件，根据文件后缀区别是word或者excel
        /// </summary>
        public string TemplateFile { get; set; }

        // public List<ReportValueMap> ValueMaps { get; set; }
        //值的匹配关系内容，是上面内容的序列号结果,每一个序列号对应一组由{Serial,TestProject,TestItem}三个参数确定的数据
        public string ValueMaps { get; set; }

    }

    /// <summary>
    /// 报告和数据的映射关系
    /// 以Excel的形式，定位报告的坐标点与数据坐标点的关系
    /// </summary>
    public class ReportValueMap
    {
        //以下两项唯一定位一份产品的测试数据
        /// <summary>
        /// 测试项目，比如温循、老练等
        /// </summary>
        public string TestProject { get; set; }
        /// <summary>
        /// 指标项目，或者称为数据项的名称
        /// </summary>
        public string DataName { get; set; }

        List<RowColIndexMap> ColIndexMap { get; set; }

        public ReportValueMap()
        {
            ColIndexMap = new List<RowColIndexMap>();
        }
    }

    public struct RowColIndexMap
    {
        //起个名字，方便记忆，可以不填
        public string Name { get; set; }
        /// <summary>
        /// 报告模板文件中的第几个表格，一般是对word模板起作用
        /// </summary>
        public uint TableIndex { get; set; }
        //报告的行列坐标
        public uint ReportRowIndex { get; set; }
        public uint ReportColIndex { get; set; }
        //数据的行列坐标
        public uint ValueRowIndex { get; set; }
        public uint ValueColIndex { get; set; }
    }
}
